<?php

use CRM_FormProcessor_ExtensionUtil as E;

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */
class CRM_FormProcessor_BAO_FormProcessorAction extends CRM_FormProcessor_DAO_FormProcessorAction {

  /**
   * Function to get values
   *
   * @return array $result found rows with data
   * @access public
   * @static
   */
  public static function getValues($params) {
    $actionProvider = form_processor_get_action_provider();

    $result = [];
    $action = new CRM_FormProcessor_BAO_FormProcessorAction();
    if (!empty($params)) {
      $fields = self::fields();
      foreach ($params as $key => $value) {
        if (isset($fields[$key])) {
          $action->$key = $value;
        }
      }
    }
    $action->orderBy('weight');
    $action->find();
    while ($action->fetch()) {
      $row = [];
      self::storeValues($action, $row);
      if (!empty($row['form_processor_instance_id'])) {
        if (isset($row['configuration']) && is_string($row['configuration'])) {
          $row['configuration'] = json_decode($row['configuration'], TRUE);
        }
        if (empty($row['configuration']) || !is_array($row['configuration'])) {
          $row['configuration'] = array();
        }

        if (isset($row['mapping']) && is_string($row['mapping'])) {
          $row['mapping'] = json_decode($row['mapping'], TRUE);
        }
        if (empty($row['mapping']) || !is_array($row['mapping'])) {
          $row['mapping'] = [];
        }

        if (isset($row['condition_configuration']) && is_string($row['condition_configuration'])) {
          $row['condition_configuration'] = json_decode($row['condition_configuration'], TRUE);
        }
        if (empty($row['condition_configuration']) || !is_array($row['condition_configuration'])) {
          $row['condition_configuration'] = null;
        }
        if (isset($row['condition_configuration']) && $row['condition_configuration']) {
          $conditionClass = $actionProvider->getConditionByName($row['condition_configuration']['name']);
          $row['condition_configuration']['title'] = $conditionClass->getTitle();
        }
        if (isset($row['delay_configuration']) && is_string($row['delay_configuration'])) {
          $row['delay_configuration'] = json_decode($row['delay_configuration'], TRUE);
        }
        if (empty($row['delay_configuration']) || !is_array($row['delay_configuration'])) {
          $row['delay_configuration'] = null;
        }

        $result[$row['id']] = $row;
      }
      else {
        //invalid input because no there is no form processor
        CRM_FormProcessor_BAO_FormProcessorAction::deleteWithId($row['id']);
      }
    }
    return $result;
  }

  /**
   * Function to add or update form processor action
   *
   * @param array $params
   *
   * @return array $result
   * @access public
   * @throws Exception when params is empty
   * @static
   */
  public static function add($params) {
    $result = [];
    if (empty($params)) {
      throw new Exception('Params can not be empty when adding or updating a form processor action');
    }

    if (!empty($params['id'])) {
      CRM_Utils_Hook::pre('edit', 'FormProcessorAction', $params['id'], $params);
    }
    else {
      CRM_Utils_Hook::pre('create', 'FormProcessorAction', NULL, $params);
    }

    $action = new CRM_FormProcessor_BAO_FormProcessorAction();
    $fields = self::fields();
    foreach ($params as $key => $value) {
      if (isset($fields[$key])) {
        $action->$key = $value;
      }
    }

    if (is_array($action->configuration)) {
      $action->configuration = json_encode($action->configuration);
    }
    if (is_array($action->mapping)) {
      $action->mapping = json_encode($action->mapping);
    }
    if (is_array($action->condition_configuration)) {
      // Delete condition if set to "No condition".
      if ($action->condition_configuration['name']) {
        $action->condition_configuration = json_encode($action->condition_configuration);
      }
      else {
        $action->condition_configuration = '';
      }
    }
    if (is_array($action->delay_configuration)) {
      $action->delay_configuration = json_encode($action->delay_configuration);
    }
    $action->save();
    self::storeValues($action, $result);

    if (!empty($params['id'])) {
      CRM_Utils_Hook::post('edit', 'FormProcessorAction', $action->id, $action);
    }
    else {
      CRM_Utils_Hook::post('create', 'FormProcessorAction', $action->id, $action);
    }

    return $result;
  }

  /**
   * Function to delete a form processor action with id
   *
   * @param int $id
   *
   * @throws Exception when $id is empty
   * @access public
   * @static
   */
  public static function deleteWithId($id) {
    if (empty($id)) {
      throw new Exception('id can not be empty when attempting to delete a form processor action');
    }

    CRM_Utils_Hook::pre('delete', 'FormProcessorAction', $id, CRM_Core_DAO::$_nullArray);

    $action = new CRM_FormProcessor_BAO_FormProcessorAction();
    $action->id = $id;
    if ($action->find(TRUE)) {
      $action->delete();
    }

    CRM_Utils_Hook::post('delete', 'FormProcessorAction', $id, CRM_Core_DAO::$_nullArray);

    return;
  }

  /**
   * Function to delete all actions with a form processor instance id
   *
   * @param int $formProcessorInstanceId
   *
   * @access public
   * @static
   */
  public static function deleteWithFormProcessorInstanceId($formProcessorInstanceId) {
    $action = new CRM_FormProcessor_BAO_FormProcessorAction();
    $action->form_processor_instance_id = $formProcessorInstanceId;
    $action->find(FALSE);
    while ($action->fetch()) {
      self::deleteWithId($action->id);
    }
  }

  /**
   * Public function to generate name from title
   *
   * @param $title
   *
   * @return string
   * @access public
   * @static
   */
  public static function buildNameFromTitle($title) {
    $titleParts = explode(' ', strtolower($title));
    $nameString = implode('_', $titleParts);
    return substr($nameString, 0, 80);
  }

  /**
   * Returns whether the name is valid or not
   *
   * @param string $name
   * @param int $form_processor_instance_id
   * @param int $id optional
   *
   * @return bool
   * @static
   */
  public static function isNameValid($name, $form_processor_instance_id, $id = NULL) {
    if (empty($form_processor_instance_id)) {
      return true;
    }
    $sql = "SELECT COUNT(*) FROM `civicrm_form_processor_action` WHERE `name` = %1 AND `form_processor_instance_id` = %2";
    $params[1] = [$name, 'String'];
    $params[2] = [$form_processor_instance_id, 'Integer'];
    if ($id) {
      $sql .= " AND `id` != %3";
      $params[3] = [$id, 'Integer'];
    }
    $count = CRM_Core_DAO::singleValueQuery($sql, $params);
    return ($count > 0) ? FALSE : TRUE;
  }


}