<?php
/**
 * @author Jaap Jansma <jaap.jansma@civicoop.org>
 * @license AGPL-3.0
 */

use CRM_FormProcessor_ExtensionUtil as E;

class CRM_FormProcessor_Form_RunFormProcessor extends CRM_Core_Form {

  protected $defaultFields = [];

  protected $fields = [];

  protected $formProcessorName;

  /**
   * Preprocess form.
   *
   * This is called before buildForm. Any pre-processing that
   * needs to be done for buildForm should be done here.
   *
   * This is a virtual function and should be redefined if needed.
   */
  public function preProcess() {
    parent::preProcess(); // TODO: Change the autogenerated stub
    $this->formProcessorName = CRM_Utils_Request::retrieve('_qf_formProcessorName', 'String', $this, TRUE);
  }

  /**
   * This virtual function is used to build the form.
   *
   * It replaces the buildForm associated with QuickForm_Page. This allows us
   * to put preProcess in front of the actual form building routine
   */
  public function buildQuickForm() {
    $this->add('hidden', '_qf_formProcessorName');
    $this->setDefaults(['_qf_formProcessorName' => $this->formProcessorName]);
    $formProcessor = civicrm_api3('FormProcessorInstance', 'getsingle', ['name' => $this->formProcessorName]);
    $defaultFields = civicrm_api3('FormProcessorDefaults', 'getfields', ['api_action' => $this->formProcessorName]);
    $this->defaultFields = $defaultFields['values'];
    $defaultFieldNames = [];
    foreach($this->defaultFields as $field) {
      $attributes = [
        'style' => 'min-width:250px',
        'class' => 'huge'
      ];
      if (isset($field['options']) && is_array($field['options'])) {
        $attributes['class'] .= ' crm-select2';
        $attributes['placeholder'] = E::ts('- select -');
        if ($field['formprocessor.is_multiple']) {
          $attributes['multiple'] = 'multiple';
        }
        $this->add('select', $this->formProcessorName . '_default_' . $field['name'], $field['title'], $field['options'], $field['api.required'], $attributes);
      } else {
        $this->add('text', $this->formProcessorName . '_default_' . $field['name'], $field['title'], $attributes, $field['api.required']);
      }
      $defaultFieldNames[] = $this->formProcessorName.'_default_'.$field['name'];
    }
    $this->assign('enableDefault', $formProcessor['enable_default_data']);
    $this->assign('defaultFieldNames', $defaultFieldNames);

    $fields = civicrm_api3('FormProcessor', 'getfields', ['api_action' => $this->formProcessorName]);
    $this->fields = $fields['values'];
    $fieldNames = [];
    foreach($this->fields as $field) {
      $attributes = [
        'style' => 'min-width:250px',
        'class' => 'huge'
      ];
      if (isset($field['options']) && is_array($field['options'])) {
        $attributes['class'] .= ' crm-select2';
        $attributes['placeholder'] = E::ts('- select -');
        if ($field['formprocessor.is_multiple']) {
          $attributes['multiple'] = 'multiple';
        }
        $this->add('select', $field['name'], $field['title'], $field['options'], $field['api.required'], $attributes);
      } else {
        $this->add('text', $field['name'], $field['title'], $attributes, $field['api.required']);
      }
      $fieldNames[] = $field['name'];
    }
    $this->assign('fieldNames', $fieldNames);

    $this->addButtons([
      [
        'type' => 'refresh',
        'name' => E::ts('Load'),
        'isDefault' => FALSE,
      ],
      [
        'type' => 'submit',
        'name' => E::ts('Submit'),
        'isDefault' => FALSE,
      ],
    ]);

    $refreshButton = false;
    if (isset($this->_submitValues['_qf_RunFormProcessor_refresh']) && $this->_submitValues['_qf_RunFormProcessor_refresh'] == E::ts('Load')) {
      // Before CiviCRM version 5.31 the submit button value was the label of the button.
      $refreshButton = true;
    } elseif (isset($this->_submitValues['_qf_RunFormProcessor_refresh']) && $this->_submitValues['_qf_RunFormProcessor_refresh'] == '1') {
      // CiviCRM version 5.31 onwards set a '1' value on the submit button.
      $refreshButton = true;
    }
    if ($refreshButton) {
      $this->loadDefaultData($this->formProcessorName);
    }
  }

  protected function loadDefaultData($formProcessorName) {
    $defaultParams = [];
    foreach($this->defaultFields as $field) {
      $submitFieldName = $formProcessorName.'_default_'.$field['name'];
      if (isset($this->_submitValues[$submitFieldName])) {
        $defaultParams[$field['name']] = $this->_submitValues[$submitFieldName];
      }
    }

    $defaultParams['version'] = 3;
    $default = civicrm_api('FormProcessorDefaults', $formProcessorName, $defaultParams);
    if (!isset($default['is_error']) || !$default['is_error']) {
      $this->setDefaults($default);
      foreach($default as $k=>$v) {
        $idx = $this->_elementIndex[$k];
        $this->_elements[$idx]->setValue($v);
      }
    } else {
      CRM_Core_Session::setStatus($default['error_message']);
      $this->clearForm();
    }
  }

  protected function clearForm() {
    foreach($this->fields as $field) {
      $idx = $this->_elementIndex[$field['name']];
      $this->_elements[$idx]->setValue('');
    }
  }

  /**
   * Performs the server side validation.
   *
   * @return bool
   *   true if no error found
   * @throws    HTML_QuickForm_Error
   * @since     1.0
   */
  public function validate() {
    $return = parent::validate();

    $submitButton = false;
    if (isset($this->_submitValues['_qf_RunFormProcessor_submit']) && $this->_submitValues['_qf_RunFormProcessor_submit'] == E::ts('Submit')) {
      // Before CiviCRM version 5.31 the submit button value was the label of the button.
      $submitButton = true;
    } elseif (isset($this->_submitValues['_qf_RunFormProcessor_submit']) && $this->_submitValues['_qf_RunFormProcessor_submit'] == '1') {
      // CiviCRM version 5.31 onwards set a '1' value on the submit button.
      $submitButton = true;
    }
    $refreshButton = false;
    if (isset($this->_submitValues['_qf_RunFormProcessor_refresh']) && $this->_submitValues['_qf_RunFormProcessor_refresh'] == E::ts('Load')) {
      // Before CiviCRM version 5.31 the submit button value was the label of the button.
      $refreshButton = true;
    } elseif (isset($this->_submitValues['_qf_RunFormProcessor_refresh']) && $this->_submitValues['_qf_RunFormProcessor_refresh'] == '1') {
      // CiviCRM version 5.31 onwards set a '1' value on the submit button.
      $refreshButton = true;
    }

    if ($refreshButton) {
      foreach($this->fields as $field) {
        if (isset($this->_errors[$field['name']])) {
          unset($this->_errors[$field['name']]);
        }
      }
    } elseif ($submitButton) {
      foreach($this->defaultFields as $field) {
        $submitFieldName = $this->formProcessorName.'_default_'.$field['name'];
        if (isset($this->_errors[$submitFieldName])) {
          unset($this->_errors[$submitFieldName]);
        }
      }
    }
    return (0 == count($this->_errors));
  }

  /**
   * Called after the form is validated.
   *
   * Any processing of form state etc should be done in this function.
   * Typically all processing associated with a form should be done
   * here and relevant state should be stored in the session
   *
   * This is a virtual function and should be redefined if needed
   */
  public function postProcess() {
    $submitted = false;
    if (isset($this->_submitValues['_qf_RunFormProcessor_submit']) && $this->_submitValues['_qf_RunFormProcessor_submit'] == E::ts('Submit')) {
      // Before CiviCRM version 5.31 the submit button value was the label of the button.
      $submitted = true;
    } elseif (isset($this->_submitValues['_qf_RunFormProcessor_submit']) && $this->_submitValues['_qf_RunFormProcessor_submit'] == '1') {
      // CiviCRM version 5.31 onwards set a '1' value on the submit button.
      $submitted = true;
    }
    if ($submitted && !count($this->_errors)) {
      $params = [];
      foreach($this->fields as $field) {
        $submitFieldName = $field['name'];
        if (isset($this->_submitValues[$submitFieldName])) {
          $params[$field['name']] = $this->_submitValues[$submitFieldName];
        }
      }

      $params['version'] = 3;
      $result = civicrm_api('FormProcessor', $this->formProcessorName, $params);
      $this->assign('result_json', json_encode($result, JSON_PRETTY_PRINT|JSON_UNESCAPED_SLASHES));
      if (!isset($result['is_error']) || !$result['is_error']) {
        $this->clearForm();
      }
    }
  }


}
