<?php
/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */

 namespace Civi\FormProcessor\API;

 use Civi\ActionProvider\Action\AbstractAction;
 use Civi\ActionProvider\Parameter\ParameterBag;
 use Civi\API\Event\ResolveEvent;
 use Civi\API\Event\RespondEvent;
 use Civi\API\Events;
 use Civi\API\Provider\ProviderInterface as API_ProviderInterface;
 use Civi\FormProcessor\DataBag;
 use Civi\FormProcessor\Type\OptionListInterface;
 use Civi\FormProcessor\Utils\Cache;
 use Symfony\Component\EventDispatcher\EventSubscriberInterface;

 use CRM_FormProcessor_ExtensionUtil as E;

/**
 * This class connects the enabled form processors to
 * the the api interface of CiviCRM. That way the form processors
 * could be executed over the api.
 */
 class FormProcessor implements API_ProviderInterface, EventSubscriberInterface {

  public function __construct() {
  }

  /**
   * @return array
   */
  public static function getSubscribedEvents() {
    // Some remarks on the solution to implement the getFields method.
    // We would like to have implemented it as a normal api call. Meaning
    // it would been processed in the invoke method.
    //
    // However the reflection provider has a stop propegation so we cannot use
    // getfields here unles we are earlier then the reflection provider.
    // We should then use a weight lower than Events::W_EARLY and do a
    // stop propegation in our event. But setting an early flag is not a neat way to do stuff.
    // So instead we use the Respond event and check in the respond event whether the action is getFields and
    // if so do our getfields stuff there.
    return array(
      Events::RESOLVE => array(
        array('onApiResolve', Events::W_EARLY),
      ),
    );
  }

  public function onApiResolve(ResolveEvent $event) {
    $apiRequest = $event->getApiRequest();
    if (strtolower($apiRequest['entity']) == 'formprocessor') {
      $event->setApiProvider($this);
      if (strtolower($apiRequest['action']) == 'getfields' || strtolower($apiRequest['action']) == 'getoptions') {
        $event->stopPropagation();
      }
    }
  }

  /**
   * Event listener on the ResponddEvent to handle the getfields actions.
   * So the fields defined by the user are availble in the api explorer for example.
   */
  public function invokeGetFields($apiRequest) {
    $params = $apiRequest['params'];
    $result = array();

    // Now check whether the action param is set. With the action param we can find the form processor.
    if (isset($params['action'])) {
      $result['values'] = $this->getFields($params['action'], $params);
      if (!is_array($result['values'])) {
        $result['values'] = array();
      }
      $result['count'] = count($result['values']);
    }
    return $result;
  }

  protected function getFields($formProcessorName, $apiParams, $fieldName = null, $useCache=true) {
    $fields = array();
    $hash = md5(json_encode($apiParams));
    $cacheKey = 'FormProcessor.getfields.'.strtolower($formProcessorName).'.'.$hash;
    if ($useCache && ($cache = Cache::get($cacheKey))) {
      return $cache;
    }

    try {
      $formProcessor = self::getFormProcessor($formProcessorName);
    } catch (\API_Exception $e) {
      return;
    }

    // Process all inputs of the formprocessor.
    foreach($formProcessor['inputs'] as $input) {
      if ($fieldName && $fieldName != $input['name']) {
        continue;
      }
      $field = array(
        'name' => $input['name'],
        'title' => $input['title'],
        'description' => '',
        'type' => $input['type']->getCrmType(),
        'api.required' => $input['is_required'],
        'api.aliases' => array(),
        'entity' => 'FormProcessor',
      );
      if ($input['type'] instanceof OptionListInterface) {
        $field['options'] = $input['type']->getOptions($apiParams);
        $field['formprocessor.is_multiple'] = $input['type']->isMultiple(); // We add our own spec here, as an option for is multiple does not exists.
      }
      // Set a default value
      if (isset($input['default_value']) && $input['default_value'] != '') {
        $field['api.default'] = $input['type']->getDefaultValue($input['default_value']);
      }
      if ($input['type']->getCrmType()) {
        $fields[$input['name']] = $field;
      }
    }
    Cache::set($cacheKey, $fields);
    return $fields;
  }

   /**
    * Invoke the GetOptions api call
    *
    * @param $apiRequest
    * @return array
    */
   protected function invokeGetOptions($apiRequest) {
     $params = $apiRequest['params'];
     $result = array();
     $result['values'] = array();
     // Now check whether the action param is set. With the action param we can find the data processor.
     if (isset($params['field'])) {
       try {
         $fieldName = $params['field'];
         $fields = $this->getFields($params['api_action'], $params, $fieldName, false);
         if (isset($fields[$fieldName]) && isset($fields[$fieldName]['options'])) {
           $result['values'] = $fields[$fieldName]['options'];
         }
       } catch(\Exception $e) {
         // Do nothing.
       }

       $result['count'] = count($result['values']);
     }
     return $result;
   }

   /**
    * @param array $apiRequest
    *   The full description of the API request.
    * @return array
    *   structured response data (per civicrm_api3_create_success)
    * @see civicrm_api3_create_success
    * @throws \Exception
    */
   public function invoke($apiRequest) {
     switch (strtolower($apiRequest['action'])) {
       case 'getfields':
         // Do get fields
         return $this->invokeGetFields($apiRequest);
         break;
       case 'getoptions':
         // Do get options
         return $this->invokeGetOptions($apiRequest);
         break;
       default:
         return $this->invokeFormProcessor($apiRequest);
         break;
     }
   }

   /**
    * Gets the form processor from the cache.
    * @param $name
    *
    * @return mixed
    * @throws \API_Exception
    * @throws \CRM_Core_Exception
    */
   protected static function getFormProcessor($name) {
     $cachekey = 'FormProcessor.run.get.'.$name;
     if (!$formProcessor = Cache::get($cachekey)) {
       // Find the form processor
       $formProcessors = \CRM_FormProcessor_BAO_FormProcessorInstance::getValues(['name' => $name]);
       if (count($formProcessors) != 1) {
         throw new \API_Exception('Could not find a form processor');
       }
       $formProcessor = reset($formProcessors);
       Cache::set($cachekey, $formProcessor);
     }
     return $formProcessor;
   }

  /**
   * @param array $apiRequest
   *   The full description of the API request.
   * @return array
   *   structured response data (per civicrm_api3_create_success)
   * @see civicrm_api3_create_success
   * @throws \API_Exception
   */
  public function invokeFormProcessor($apiRequest) {
    $container = \Civi::container();
    /*
     * @var Civi\FormProcessor\DelayedAction\Factory
     */
    $delayedActionFactory = $container->get('form_processor_delayed_action_factory');

    $actionProvider = form_processor_get_action_provider();
    $dataBag = new DataBag();

    $params = $apiRequest['params'];

    $formProcessor = self::getFormProcessor($apiRequest['action']);


    // Validate the parameters.
    foreach($formProcessor['inputs'] as $input) {
      $inputType = $input['type'];
      unset($input['type']);
      $objInput = new \CRM_FormProcessor_BAO_FormProcessorInput();
      $objInput->copyValues($input);

      // Set default value
      if (!isset($params[$input['name']]) && isset($input['default_value']) && $input['default_value'] != '') {
        $params[$input['name']] = $inputType->getDefaultValue($input['default_value']);
      }

      if ($input['is_required'] && !isset($params[$input['name']])) {
        throw new \API_Exception('Parameter '.$input['name'].' is required');
      }
      if (isset($params[$input['name']]) && !empty($params[$input['name']]) && !$inputType->validateValue($params[$input['name']], $params)) {
        throw new \API_Exception('Parameter '.$input['name'].' is invalid');
      }
      // Check the validations on the input.
      if (isset($params[$input['name']]) && $params[$input['name']] != "") {
        foreach ($input['validators'] as $validator) {
          if (!$validator['validator']->validate($params[$input['name']], $inputType)) {
            throw new \API_Exception($validator['validator']->getInvalidMessage() . ' (Parameter ' . $input['name'] . ')');
          }
        }
      }
      if (isset($params[$objInput->name])) {
        $dataBag->setInputData($objInput, $inputType->normalizeValue($params[$input['name']]));
      }
    }

    // Execute the actions
    foreach($formProcessor['actions'] as $action) {
      $actionClass = $actionProvider->getActionByName($action['type']);
      $actionClass->getConfiguration()->fromArray($action['configuration']);
      $objAction = new \CRM_FormProcessor_BAO_FormProcessorAction();
      $objAction->copyValues($action);

      // Create a parameter bag for the action
      $mappedParameterBag = $this->convertDataBagToMappedParameterBag($dataBag, $action['mapping'], $actionProvider);

      // Create a condition class for this action
      if (!is_array($action['condition_configuration'])) {
        $action['condition_configuration'] = array();
      }
      if (!isset($action['condition_configuration']['parameter_mapping']) || !is_array($action['condition_configuration']['parameter_mapping'])) {
        $action['condition_configuration']['parameter_mapping'] = array();
      }
      $mappedConditionParameterBag = $this->convertDataBagToMappedParameterBag($dataBag, $action['condition_configuration']['parameter_mapping'], $actionProvider);
      if (!isset($action['condition_configuration']['output_mapping']) || !is_array($action['condition_configuration']['output_mapping'])) {
        $action['condition_configuration']['output_mapping'] = array();
      }
      $mappedConditionOutputParameterBag = $this->convertDataBagToMappedParameterBag($dataBag, $action['condition_configuration']['output_mapping'], $actionProvider);
      $condition = \CRM_FormProcessor_Condition::getConditionClass($action['condition_configuration']);
      $actionClass->setCondition($condition);

      // Check whether the action should be delayed
      if (isset($action['delay']) && $action['delay']) {
        $delayClass = $delayedActionFactory->getHandlerByName($action['delay']);
        $configuration = $delayClass->getDefaultConfiguration();
        if (is_array($action['delay_configuration'])) {
          foreach ($action['delay_configuration'] as $name => $value) {
            $configuration->set($name, $value);
          }
        }
        $delayClass->setConfiguration($configuration);
        $delayTo = $delayClass->getDelayedTo($dataBag);
        $this->queueAction($formProcessor['name'], $action['name'], $mappedParameterBag, $mappedConditionParameterBag, $delayTo);
      } else {
        // There is no delay. Execute the action immediately
        // Now execute the action
        try {
          $outputBag = $actionClass->execute($mappedParameterBag, $mappedConditionParameterBag, $mappedConditionOutputParameterBag);
          // Add the output of the action to the data bag of this action.
          $dataBag->setActionDataFromActionProviderParameterBag($objAction, $outputBag);
        } catch (\Exception $e) {
          throw new Exception($action['title'], $e);
        }
      }
    }

    return $formProcessor['output_handler']->handle($dataBag);
  }

   /**
    * Process delayed actions
    *
    * @param int $maxRunTime
    * @return array
    */
   public static function processDelayedActions($maxRunTime=30) {
     $queue = \CRM_Queue_Service::singleton()->create(array(
       'type' => 'FormProcessor',
       'name' => 'FormProcessorDelayedActions',
       'reset' => false, //do not flush queue upon creation
     ));

     $returnValues = array();

     //retrieve the queue
     $runner = new \CRM_Queue_Runner(array(
       'title' => E::ts('Process delayed form processor actions'), //title fo the queue
       'queue' => $queue, //the queue object
       'errorMode'=> \CRM_Queue_Runner::ERROR_CONTINUE, //continue on error otherwise the queue will hang
     ));

     $stopTime = time() + $maxRunTime; //stop executing next item after 30 seconds
     while((time() < $stopTime) && $queue->numberOfItems() > 0) {
       $result = $runner->runNext(false);
       $returnValues[] = $result;

       if (!$result['is_continue']) {
         break;
       }
     }

     return $returnValues;
   }

   /**
    * Executes a delayed action.
    *
    * @param $form_processor_name
    * @param $action_name
    * @param $mappedParameterBag
    * @param $mappedConditionParameterBag
    */
  public static function executeDelayedAction(\CRM_Queue_TaskContext $context, $form_processor_name, $action_name, $parameters, $conditionParameters) {
    $actionProvider = form_processor_get_action_provider();

    $conditionOutputBag = new ParameterBag();
    $mappedParameters = new ParameterBag();
    foreach($parameters as $name => $val) {
      $mappedParameters->setParameter($name, $val);
    }
    $mappedConditionParameters = new ParameterBag();
    foreach($conditionParameters as $name => $val) {
      $mappedConditionParameters->setParameter($name, $val);
    }

    // Find the form processor
    try {
      $formProcessor = self::getFormProcessor($form_processor_name);
    } catch (\API_Exception $e) {
      return TRUE;
    }
    // FInd the action
    foreach($formProcessor['actions'] as $action) {
      if ($action['name'] == $action_name) {
        $actionClass = $actionProvider->getActionByName($action['type']);
        $actionClass->getConfiguration()->fromArray($action['configuration']);

        // Create a condition class for this action
        if (!isset($action['condition_configuration']) || !is_array($action['condition_configuration'])) {
          $action['condition_configuration'] = array();
        }
        $condition = \CRM_FormProcessor_Condition::getConditionClass($action['condition_configuration']);
        $actionClass->setCondition($condition);

        // Execute the action
        $actionClass->execute($mappedParameters, $mappedConditionParameters, $conditionOutputBag);
      }
    }
    return true;
  }

   /**
    * Save a delayed action into the queue
    *
    * @param $form_processor_name
    * @param $action_name
    * @param $mappedParameterBag
    * @param $mappedConditionParameterBag
    * @param \DateTime $delayedTo
    */
  protected function queueAction($form_processor_name, $action_name, ParameterBag $mappedParameterBag, ParameterBag $mappedConditionParameterBag, \DateTime $delayedTo) {
    $queue = \CRM_Queue_Service::singleton()->create(array(
      'type' => 'FormProcessor',
      'name' => 'FormProcessorDelayedActions',
      'reset' => false, //do not flush queue upon creation
    ));

    //create a task with the action and eventData as parameters
    $task = new \CRM_Queue_Task(
      array('Civi\FormProcessor\API\FormProcessor', 'executeDelayedAction'), //call back method
      array($form_processor_name, $action_name, $mappedParameterBag->toArray(), $mappedConditionParameterBag->toArray()),
      'Delayed Form Processor Action: '.$form_processor_name.'.'.$action_name
    );

    //save the task with a delay
    $dao              = new \CRM_Queue_DAO_QueueItem();
    $dao->queue_name  = $queue->getName();
    $dao->submit_time = \CRM_Utils_Time::getTime('YmdHis');
    $dao->data        = serialize($task);
    $dao->weight      = 0; //weight, normal priority
    $dao->release_time = $delayedTo->format('YmdHis');
    $dao->save();
  }

  /**
   * Converts a DataBag object to a mapped parameterBag.
   *
   * @param DataBag $dataBag
   * @param array $mapping
   * @param $actionProvider
   * @return ParameterBag;
   */
  protected function convertDataBagToMappedParameterBag(DataBag $dataBag, $mapping, $actionProvider) {
    $parameterBag = $actionProvider->createParameterBag();
    $fields = $dataBag->getAllAliases();
    foreach($fields as $field) {
      $parameterBag->setParameter($field, $dataBag->getDataByAlias($field));
    }
    $mappedParameterBag = $actionProvider->createdMappedParameterBag($parameterBag, $mapping);
    return $mappedParameterBag;
  }

  /**
   * @param int $version
   *   API version.
   * @return array<string>
   */
  public function getEntityNames($version) {
    return array(
      'FormProcessor'
    );
  }

  /**
   * @param int $version
   *   API version.
   * @param string $entity
   *   API entity.
   * @return array<string>
   */
  public function getActionNames($version, $entity) {
    if (strtolower($entity) != 'formprocessor') {
      return array();
    }

    $cachekey = 'formprocessor.getactions';
    if ($cache = Cache::get($cachekey)) {
      return $cache;
    }

    $params['is_active'] = 1;
    $form_processors = \CRM_FormProcessor_BAO_FormProcessorInstance::getValues($params);
    $actions[] = 'getfields';
    $actions[] = 'getoptions';
    foreach($form_processors as $form_processor) {
      $actions[] = $form_processor['name'];
    }
    Cache::set($cachekey, $actions);
    return $actions;
  }

 }
